#ifndef MPLAYER_H
#define MPLAYER_H

#include <QWidget>
class QTimer;
class QMenu;
class QProcess;
class QResizeEvent;

typedef enum
{
    STOPPING,
    VIDEO_PLAYING,
    VIDEO_PAUSING
} MPlayerState;



class MPlayer : public QWidget
{
    Q_OBJECT

signals:
    void played(void);
    void paused(void);
    void stopped(void);
    void catching(bool);
    void fullScreen(void);
    void timeChanged(int pos);
    void lengthChanged(int len);

public:
    explicit MPlayer(QWidget *parent = 0);
    MPlayerState state;
    static const int UPDATE_FREQUENCY = 10; //Update the progress every 10s.
    inline QWidget* getLayer(){return layer;}

public slots:
    void stop(void);
    void changeState(void);
    void setProgress(int pos);
    void setVolume(int percentage);
    void openFile(QString&);
    void setRatio_16_9(void);
    void setRatio_16_10(void);
    void setRatio_4_3(void);
    void setRatio_0(void);

protected:
    void resizeEvent(QResizeEvent *);
    void mouseDoubleClickEvent(QMouseEvent *);

private slots:
    void onFinished(int);
    void readOutput(void);
    void updateTime(void);
    void showMenu(const QPoint&);

private:
    QProcess* process;
    QWidget* layer;
    QTimer* timer;
    QTimer* vol_timer;
    QMenu* menu;
    int progress;
    int length;
    int w, h;
    bool is_waiting;
    bool is_caching;
    QString wait_to_play;

    void cb_start(QString& msg);
    void cb_ratioChanged(QString& msg);
    void cb_updateTime(QString& msg);
    void resizeLayer(void);
};

#endif // MPLAYER_H
