#include "playlist.h"
#include "ui_playlist.h"
#include <QFileDialog>
#include <QInputDialog>
#include <QTimer>
#include <QListWidget>

Playlist::Playlist(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::Playlist)
{
    ui->setupUi(this);
    connect(ui->addButton, SIGNAL(clicked()), this, SLOT(onAddButton()));
    connect(ui->delButton, SIGNAL(clicked()), this, SLOT(onDelButton()));
    connect(ui->netButton, SIGNAL(clicked()), this, SLOT(onNetButton()));
    connect(ui->clearButton, SIGNAL(clicked()), this, SLOT(onClearButton()));
    connect(ui->listWidget, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(selectFile(QListWidgetItem*)));
}

Playlist::~Playlist()
{
    delete ui;
}

void Playlist::onDelButton()
{
    if (ui->listWidget->count() == 0)
        return;
    int row = ui->listWidget->currentRow();
    QListWidgetItem* item = ui->listWidget->takeItem(row);
    delete item;
    filelist.remove(row);
}

void Playlist::onClearButton()
{
    ui->listWidget->clear();
    filelist.clear();
}

void Playlist::onAddButton()
{
    emit needPause(true);

    QStringList files = QFileDialog::getOpenFileNames(this);
    while (!files.isEmpty())
    {
        QString file = files.takeFirst();
        QString name = file.section('/', -1, -1);
        addFile(name, file);
    }

    emit needPause(false);
}

void Playlist::addFile(QString& name, QString& file)
{
    filelist.append(file);
    ui->listWidget->addItem(new QListWidgetItem(name));
}

void Playlist::addFileAndPlay(QString& name, QString& file)
{
    last_index = filelist.size();
    filelist.append(file);
    ui->listWidget->addItem(new QListWidgetItem(name));
    emit fileSelected(file);
}

void Playlist::onNetButton()
{
    bool isok;
    emit needPause(true);
    QString url = QInputDialog::getText(this, tr("Enter url"), tr("Please enter url"),
                                        QLineEdit::Normal, "", &isok);
    if (isok)
    {
        ui->listWidget->addItem("Url");
        filelist.append(url);
    }
    emit needPause(false);
}

void Playlist::selectFile(QListWidgetItem *item)
{
    int i = ui->listWidget->row(item);
    last_index = i;
    emit fileSelected(filelist[i]);
}

void Playlist::playNext()
{
    last_index++;
    if (last_index < filelist.size())
        emit fileSelected(filelist[last_index]);
}
