#include "webvideo.h"
#include "ui_webvideo.h"
#include <QMessageBox>
#include <QNetworkAccessManager>
#include <QNetworkRequest>
#include <QNetworkReply>
#include <QTextCodec>
#include <QMessageBox>
#include <QProcess>
#include <QDir>
#include <QHttp>

//Download file
Downloader::Downloader(QObject *parent) : QObject(parent)
{
    wget = new QProcess(this);
    wget->setProcessChannelMode(QProcess::MergedChannels);
    connect(wget, SIGNAL(readyReadStandardOutput()), this, SLOT(onReadyRead()));
    connect(wget, SIGNAL(finished(int)), this, SLOT(onFinished(int)));
}

void Downloader::addTask(const QString &name, const QString &url)
{
    names << name;
    urls << url;
    if (wget->state() == QProcess::NotRunning) //No task is running
        start();
}

void Downloader::start()
{
    //Set downloading file's name
    downloading_file = names.takeFirst();

    //Set arguments
    QStringList args;
    args << urls.takeFirst();
    args << "-O" << QDir::home().filePath(downloading_file);
    args << "-o" << "/dev/stdout"; //Catch output to read progress
    //prevent 403 error
    args << "-U" << "Opera/9.80 (X11; Linux i686; U; Edition Next; zh-cn) Presto/2.11.310 Version/12.50";

    wget->start("wget", args, QProcess::ReadOnly);
}

void Downloader::onReadyRead()
{
    while (wget->canReadLine())
    {
        QString msg = wget->readLine();
        int end = msg.lastIndexOf('%');
        int start = msg.lastIndexOf(' ', end) + 1;
        int progress = msg.mid(start, end - start).toInt();
        emit progressChanged(progress);
    }
}

void Downloader::onFinished(int)
{
    emit finished(downloading_file, false);

    if (!names.isEmpty())
        start();
}


WebVideo::WebVideo(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::WebVideo)
{
    ui->setupUi(this);
    manager = new QNetworkAccessManager(this);
    parse_manager = new QNetworkAccessManager(this);

    //down search page and parse
    connect(ui->goButton, SIGNAL(clicked()), this, SLOT(downSearchPage()));
    connect(ui->lineEdit, SIGNAL(returnPressed()), this, SLOT(downSearchPage()));
    connect(ui->nextButton, SIGNAL(clicked()), this, SLOT(nextSearchPage()));
    connect(ui->prevButton, SIGNAL(clicked()), this, SLOT(prevSearchPage()));
    connect(manager, SIGNAL(finished(QNetworkReply*)), this, SLOT(searchVideo(QNetworkReply*)));

    //Parse video's url and play
    connect(ui->listWidget, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(startPlayOrDown(QListWidgetItem*)));
    connect(parse_manager, SIGNAL(finished(QNetworkReply*)), this, SLOT(cb_startPlayOrDown(QNetworkReply*)));

    //Download video file
    downloader = new Downloader;
    connect(downloader, SIGNAL(finished(QString&,bool)), this, SLOT(onDownDone(QString&,bool)));
    connect(downloader, SIGNAL(progressChanged(int)), this, SLOT(onDownProgress(int)));
    connect(ui->downButton, SIGNAL(clicked()), this, SLOT(onDownButton()));
    next_is_to_down = false;
}

WebVideo::~WebVideo()
{
    delete ui;
}

//Search videos
void WebVideo::downSearchPage()
{
    // get key word
    keyword = ui->lineEdit->text().simplified();
    if (keyword.isEmpty())
        return;

    // set search page url
    QString url = "http://www.soku.com/search_video/q_" + keyword;
    page_n = 1;

    // get search page
    ui->goButton->setEnabled(false);
    ui->prevButton->setEnabled(false);
    manager->get(QNetworkRequest(url));
}

void WebVideo::nextSearchPage()
{
    if (keyword.isEmpty())
        return;

    QString url = "http://www.soku.com/search_video/q_" + keyword;
    url.append("_orderby_1_page_");
    page_n++;
    url.append(QString::number(page_n));

    ui->prevButton->setEnabled(true);
    manager->get(QNetworkRequest(url));
}

void WebVideo::prevSearchPage()
{
    if (keyword.isEmpty())
        return;
    QString url = "http://www.soku.com/search_video/q_" + keyword;
    url.append("_orderby_1_page_");
    page_n--;
    url.append(QString::number(page_n));
    if (page_n == 1)
        ui->prevButton->setEnabled(false);
    manager->get(QNetworkRequest(url));
}


void WebVideo::searchVideo(QNetworkReply* reply)
{

    QTextCodec* codec = QTextCodec::codecForName("utf8");
    QString line, title, url;
    QHash<QString, int> has_read;
    bool ok;

    ui->listWidget->clear();
    result.clear();

    while (!reply->atEnd())
    {
        line = codec->toUnicode(reply->readLine());
        ok = parseYoukuSearch(line, title, url);
        if (ok)
        {
            //skip if the url has read
            if (has_read.contains(url))
                continue;
            has_read[url] = 0;

            ui->listWidget->addItem(title);
            result.append(url);
        }
    }
    reply->deleteLater();
    ui->goButton->setEnabled(true);
}

bool WebVideo::parseYoukuSearch(QString& line, QString& title, QString& url)
{
    static QRegExp htmlreg("&.+;");
    if (line.contains("<a title=") && line.contains("v.youku.com"))
    {
        title = line.section('"', 1, 1).replace(htmlreg, "");
        url = line.section('"', 5, 5);
        return true;
    }
    return false;
}

// Parse video url and play or download
void WebVideo::startPlayOrDown(QListWidgetItem *item)
{
    if (!parsing_filename.isNull()) //Other file is parsing
    {
        QMessageBox::warning(this, tr("Warning"), tr("Other file is parsing, please wait."));
        return;
    }

    int i = ui->listWidget->row(item);
    parsing_filename = item->text();
    QUrl result_page("http://www.flvcd.com/parse.php?kw=" + result[i]);
    parse_manager->get(QNetworkRequest(result_page));
}

void WebVideo::cb_startPlayOrDown(QNetworkReply *reply)
{
    QString line;
    QString name;
    QString url;
    bool first = true;
    int n = 0;

    while (!reply->atEnd())
    {
        line = reply->readLine();
        if (line.contains("<U>"))
        {
            url = line.section('>', 1, 1);

            name = parsing_filename;
            name.append('_');
            name.append(QString::number(n));
            name.append(".flv");
            n++;

            if (next_is_to_down) //Download file
                downloader->addTask(name, url);
            else if (first)
            {
                emit newUrlAndPlay(name, url);
                first = false;
            }
            else
                emit newUrl(name, url);
        }
    }
    parsing_filename.clear();
    reply->deleteLater();
    next_is_to_down = false;
}

void WebVideo::onDownButton()
{
    QListWidgetItem* item = ui->listWidget->currentItem();
    if (item == 0)
        return;
    next_is_to_down = true;
    startPlayOrDown(item);
}

void WebVideo::onDownProgress(int progress)
{
    QString msg = tr("Downloading: ");
    msg += downloader->fileName(); //Get downloading file name
    msg += ' ';
    msg += QString::number(progress); //Get progress
    msg += "%; ";
    msg += QString::number(downloader->count()); //Get the number of waiting files
    msg += tr("file(s) waiting");
    ui->downLabel->setText(msg);
}

void WebVideo::onDownDone(QString &name, bool error)
{
    if (error)
        ui->downLabel->setText(tr("Cannot download file: ") + name);
    else
        ui->downLabel->clear();
}
