#ifndef WEBVIDEO_H
#define WEBVIDEO_H

#include <QWidget>
class QNetworkAccessManager;
class QNetworkReply;
class QListWidgetItem;
class QProcess;
class QFile;

namespace Ui {
class WebVideo;
}

class Downloader : public QObject
{
    Q_OBJECT
signals:
    void finished(QString& name, bool error);
    void progressChanged(int);

public:
    Downloader(QObject* parent = 0);
    void addTask(const QString& name, const QString& url);

    inline int count(){return names.count();}
    inline const QString& fileName(){return downloading_file;}

private:
    QProcess* wget;
    QString downloading_file;
    QStringList names;
    QStringList urls;
    void start(void);

private slots:
    void onFinished(int);
    void onReadyRead(void);
};




class WebVideo : public QWidget
{
    Q_OBJECT
    
signals:
    void newUrl(QString& name, QString& url);
    void newUrlAndPlay(QString& name, QString& url);

public:
    explicit WebVideo(QWidget *parent = 0);
    ~WebVideo();
    
private:
    Ui::WebVideo *ui;

    QVector<QString> result; //search result
    QNetworkAccessManager* manager; //download search page
    QString keyword;
    int page_n;

    QNetworkAccessManager* parse_manager; //download parse page
    Downloader* downloader; //Download video files
    QString parsing_filename; //Video's name
    bool next_is_to_down;

    bool parseYoukuSearch(QString& line, QString& title, QString& url);

private slots:
    void downSearchPage(void);
    void nextSearchPage(void);
    void prevSearchPage(void);
    void searchVideo(QNetworkReply* reply);
    void startPlayOrDown(QListWidgetItem* item);
    void cb_startPlayOrDown(QNetworkReply* reply);
    void onDownButton(void);
    void onDownDone(QString& name, bool error);
    void onDownProgress(int progress);
};

#endif // WEBVIDEO_H
